from __future__ import absolute_import, division, print_function, unicode_literals

from piculet import build_tree, xpath


content = '<d><t1>foo</t1><t1 a="v"><t2>bar</t2></t1></d>'
root = build_tree(content)


def test_non_text_queries_should_return_elements():
    selected = xpath(root, './/t1')
    assert [s.tag for s in selected] == ['t1', 't1']


def test_child_text_queries_should_return_strings():
    selected = xpath(root, './/t1/text()')
    assert selected == ['foo']


def test_descendant_text_queries_should_return_strings():
    selected = xpath(root, './/t1//text()')
    assert selected == ['foo', 'bar']


def test_attr_queries_should_return_strings():
    selected = xpath(root, './/t1/@a')
    assert selected == ['v']


def test_non_absolute_queries_should_be_ok():
    selected = xpath(root, '//t1')
    assert [s.tag for s in selected] == ['t1', 't1']



from __future__ import absolute_import, division, print_function, unicode_literals

from pytest import fixture

import logging
import os
import sys
from hashlib import md5
from io import BytesIO

import piculet


PY2 = sys.version_info < (3, 0)


if PY2:
    import mock
else:
    from unittest import mock

if PY2:
    from urllib2 import urlopen
else:
    from urllib.request import urlopen


logging.raiseExceptions = False


cache_dir = os.path.join(os.path.dirname(__file__), '.cache')
if not os.path.exists(cache_dir):
    os.makedirs(cache_dir)


def mock_urlopen(url):
    key = md5(url.encode('utf-8')).hexdigest()
    cache_file = os.path.join(cache_dir, key)
    if not os.path.exists(cache_file):
        content = urlopen(url).read()
        with open(cache_file, 'wb') as f:
            f.write(content)
    else:
        with open(cache_file, 'rb') as f:
            content = f.read()
    return BytesIO(content)


piculet.urlopen = mock.Mock(wraps=mock_urlopen)


@fixture(scope='session')
def shining_content():
    """Contents of the shining.html file."""
    file_path = os.path.join(os.path.dirname(__file__), '..', 'examples', 'shining.html')
    with open(file_path) as f:
        content = f.read()
    return content


@fixture
def shining(shining_content):
    """Root element of the XML tree for the movie document "The Shining"."""
    return piculet.build_tree(shining_content)



from __future__ import absolute_import, division, print_function, unicode_literals

from pytest import raises

from piculet import extract, preprocess


def test_unknown_preprocessor_should_raise_error(shining):
    with raises(ValueError):
        pre = [{'op': 'foo', 'path': '//tr[1]'}]
        preprocess(shining, pre)


def test_remove_should_remove_selected_element(shining):
    pre = [{'op': 'remove', 'path': '//tr[1]'}]
    items = [{'key': 'cast',
              'value': {
                  'foreach': '//table[@class="cast"]/tr',
                  'items': [
                      {'key': 'name', 'value': {'path': './td[1]/a/text()'}}
                  ]
              }}]
    preprocess(shining, pre)
    data = extract(shining, items)
    assert data == {'cast': [{'name': 'Shelley Duvall'}]}


def test_remove_selected_none_should_not_cause_error(shining):
    pre = [{'op': 'remove', 'path': '//tr[50]'}]
    items = [{'key': 'cast',
              'value': {
                  'foreach': '//table[@class="cast"]/tr',
                  'items': [
                      {'key': 'name', 'value': {'path': './td[1]/a/text()'}}
                  ]
              }}]
    preprocess(shining, pre)
    data = extract(shining, items)
    assert data == {'cast': [{'name': 'Jack Nicholson'}, {'name': 'Shelley Duvall'}]}


def test_set_attr_value_from_str_should_set_attribute_for_selected_elements(shining):
    pre = [{'op': 'set_attr', 'path': "//ul[@class='genres']/li",
            'name': 'foo', 'value': "bar"}]
    items = [{'key': 'genres',
              'value': {
                  'foreach': "//li[@foo='bar']",
                  'path': "./text()"
              }}]
    preprocess(shining, pre)
    data = extract(shining, items)
    assert data == {'genres': ['Horror', 'Drama']}


def test_set_attr_value_from_path_should_set_attribute_for_selected_elements(shining):
    pre = [{'op': 'set_attr', 'path': '//ul[@class="genres"]/li',
            'name': 'foo',
            'value': {'path': './text()'}}]
    items = [{'key': 'genres',
              'value': {
                  'foreach': '//li[@foo]',
                  'path': './@foo'
              }}]
    preprocess(shining, pre)
    data = extract(shining, items)
    assert data == {'genres': ['Horror', 'Drama']}


def test_set_attr_value_from_path_no_value_should_be_ignored(shining):
    pre = [{'op': 'set_attr', 'path': '//ul[@class="genres"]/li',
            'name': 'foo',
            'value': {'path': './@bar'}}]
    items = [{'key': 'genres',
              'value': {
                  'foreach': '//li[@foo]',
                  'path': './@foo'
              }}]
    preprocess(shining, pre)
    data = extract(shining, items)
    assert data == {}


def test_set_attr_name_from_path_should_set_attribute_for_selected_elements(shining):
    pre = [{'op': 'set_attr', 'path': '//ul[@class="genres"]/li',
            'name': {'path': './text()'},
            'value': 'bar'}]
    items = [{'key': 'genres',
              'value': {
                  'foreach': '//li[@Horror]',
                  'path': './@Horror'
              }}]
    preprocess(shining, pre)
    data = extract(shining, items)
    assert data == {'genres': ['bar']}


def test_set_attr_name_from_path_no_value_should_be_ignored(shining):
    pre = [{'op': 'set_attr', 'path': '//ul[@class="genres"]/li',
            'name': {'path': './@bar'},
            'value': 'bar'}]
    items = [{'key': 'genres',
              'value': {
                  'foreach': './/li[@Horror]',
                  'path': './@Horror'
              }}]
    preprocess(shining, pre)
    data = extract(shining, items)
    assert data == {}


def test_set_attr_selected_none_should_not_cause_error(shining):
    pre = [{'op': 'set_attr', 'path': '//foo', 'name': 'foo', 'value': 'bar'}]
    items = [{'key': 'genres',
              'value': {
                  'foreach': '//li[@foo="bar"]',
                  'path': './@foo'
              }}]
    preprocess(shining, pre)
    data = extract(shining, items)
    assert data == {}


def test_set_text_value_from_str_should_set_text_for_selected_elements(shining):
    pre = [{'op': 'set_text', 'path': '//ul[@class="genres"]/li',
            "text": 'Foo'}]
    items = [{'key': 'genres',
              'value': {
                  'foreach': '//ul[@class="genres"]/li',
                  'path': './text()'
              }}]
    preprocess(shining, pre)
    data = extract(shining, items)
    assert data == {'genres': ['Foo', 'Foo']}


def test_set_text_value_from_path_should_set_text_for_selected_elements(shining):
    pre = [{'op': 'set_text', 'path': '//ul[@class="genres"]/li',
            "text": {
                'path': './text()',
                'transform': 'lower'
            }}]
    items = [{'key': 'genres',
              'value': {
                  'foreach': '//ul[@class="genres"]/li',
                  'path': './text()'
              }}]
    preprocess(shining, pre)
    data = extract(shining, items)
    assert data == {'genres': ['horror', 'drama']}


def test_set_text_no_value_should_be_ignored(shining):
    pre = [{'op': 'set_text', 'path': '//ul[@class="genres"]/li',
            'text': {'path': './@foo'}}]
    items = [{'key': 'genres',
              'value': {
                  'foreach': '//ul[@class="genres"]/li',
                  'path': './text()'
              }}]
    preprocess(shining, pre)
    data = extract(shining, items)
    assert data == {}



from __future__ import absolute_import, division, print_function, unicode_literals

from pytest import raises

from piculet import reducers, scrape, transformers


def test_no_rules_should_return_empty_result(shining_content):
    data = scrape(shining_content, {'items': []})
    assert data == {}


def test_extracted_value_should_be_reduced(shining_content):
    items = [{'key': 'title',
              'value': {'path': '//title/text()', 'reduce': 'first'}}]
    data = scrape(shining_content, {'items': items})
    assert data == {'title': 'The Shining'}


def test_default_reducer_should_be_concat(shining_content):
    items = [{'key': 'full_title',
              'value': {'path': '//h1//text()'}}]
    data = scrape(shining_content, {'items': items})
    assert data == {'full_title': 'The Shining (1980)'}


def test_added_reducer_should_be_usable(shining_content):
    reducers.register('second', lambda x: x[1])
    items = [{'key': 'year',
              'value': {'path': '//h1//text()', 'reduce': 'second'}}]
    data = scrape(shining_content, {'items': items})
    assert data == {'year': '1980'}


def test_unknown_reducer_should_raise_error(shining_content):
    with raises(ValueError):
        items = [{'key': 'year',
                  'value': {'path': '//h1//text()', 'reduce': 'foo'}}]
        scrape(shining_content, {'items': items})


def test_reduced_value_should_be_transformable(shining_content):
    items = [{'key': 'year',
              'value': {'path': '//span[@class="year"]/text()', 'transform': 'int'}}]
    data = scrape(shining_content, {'items': items})
    assert data == {'year': 1980}


def test_added_transformer_should_be_usable(shining_content):
    transformers.register('year25', lambda x: int(x) + 25)
    items = [{'key': 'year',
              'value': {'path': '//span[@class="year"]/text()', 'transform': 'year25'}}]
    data = scrape(shining_content, {'items': items})
    assert data == {'year': 2005}


def test_unknown_transformer_should_raise_error(shining_content):
    with raises(ValueError):
        items = [{'key': 'year',
                  'value': {'path': '//span[@class="year"]/text()', 'transform': 'year42'}}]
        scrape(shining_content, {'items': items})


def test_multiple_rules_should_generate_multiple_items(shining_content):
    items = [{'key': 'title',
              'value': {'path': '//title/text()'}},
             {'key': 'year',
              'value': {'path': '//span[@class="year"]/text()', 'transform': 'int'}}]
    data = scrape(shining_content, {'items': items})
    assert data == {'title': 'The Shining', 'year': 1980}


def test_item_with_no_data_should_be_excluded(shining_content):
    items = [{'key': 'title',
              'value': {'path': '//title/text()'}},
             {'key': 'foo',
              'value': {'path': '//foo/text()'}}]
    data = scrape(shining_content, {'items': items})
    assert data == {'title': 'The Shining'}


def test_multivalued_item_should_be_list(shining_content):
    items = [{'key': 'genres',
              'value': {'foreach': '//ul[@class="genres"]/li',
                        'path': './text()'}}]
    data = scrape(shining_content, {'items': items})
    assert data == {'genres': ['Horror', 'Drama']}


def test_multivalued_items_should_be_transformable(shining_content):
    items = [{'key': 'genres',
              'value': {'foreach': '//ul[@class="genres"]/li',
                        'path': './text()', 'transform': 'lower'}}]
    data = scrape(shining_content, {'items': items})
    assert data == {'genres': ['horror', 'drama']}


def test_empty_values_should_be_excluded_from_multivalued_item_list(shining_content):
    items = [{'key': 'foos',
              'value': {'foreach': '//ul[@class="foos"]/li',
                        'path': './text()'}}]
    data = scrape(shining_content, {'items': items})
    assert data == {}


def test_subrules_should_generate_subitems(shining_content):
    items = [{'key': 'director',
              'value': {
                  'items': [{'key': 'name',
                             'value': {'path': '//div[@class="director"]//a/text()'}},
                            {'key': 'link',
                             'value': {'path': '//div[@class="director"]//a/@href'}}]
              }}]
    data = scrape(shining_content, {'items': items})
    assert data == {'director': {'link': '/people/1', 'name': 'Stanley Kubrick'}}


def test_multivalued_subrules_should_generate_list_of_subitems(shining_content):
    items = [{'key': 'cast',
              'value': {
                  'foreach': '//table[@class="cast"]/tr',
                  'items': [
                      {'key': 'name',
                       'value': {'path': './td[1]/a/text()'}},
                      {'key': 'character',
                       'value': {'path': './td[2]/text()'}}
                  ]
              }}]
    data = scrape(shining_content, {'items': items})
    assert data == {'cast': [
        {'character': 'Jack Torrance', 'name': 'Jack Nicholson'},
        {'character': 'Wendy Torrance', 'name': 'Shelley Duvall'}
    ]}


def test_subitems_should_be_transformable(shining_content):
    transformers.register('stars', lambda x: '%(name)s as %(character)s' % x)
    items = [{'key': 'cast',
              'value': {
                  'foreach': '//table[@class="cast"]/tr',
                  'items': [
                      {'key': 'name', 'value': {'path': './td[1]/a/text()'}},
                      {'key': 'character', 'value': {'path': './td[2]/text()'}}
                  ],
                  'transform': 'stars'
              }}]
    data = scrape(shining_content, {'items': items})
    assert data == {'cast': ['Jack Nicholson as Jack Torrance',
                             'Shelley Duvall as Wendy Torrance']}


def test_key_should_be_generatable_using_path(shining_content):
    items = [
        {
            'foreach': '//div[@class="info"]',
            'key': {'path': './h3/text()'},
            'value': {'path': './p/text()'}
        }
    ]
    data = scrape(shining_content, {'items': items})
    assert data == {'Language:': 'English', 'Runtime:': '144 minutes'}


def test_generated_key_should_be_normalizable(shining_content):
    items = [
        {
            'foreach': '//div[@class="info"]',
            'key': {'path': './h3/text()',
                    'reduce': 'normalize'},
            'value': {'path': './p/text()'}
        }
    ]
    data = scrape(shining_content, {'items': items})
    assert data == {'language': 'English', 'runtime': '144 minutes'}


def test_generated_key_should_be_transformable(shining_content):
    items = [
        {
            'foreach': '//div[@class="info"]',
            'key': {'path': './h3/text()',
                    'reduce': 'normalize',
                    'transform': 'upper'},
            'value': {'path': './p/text()'}
        }
    ]
    data = scrape(shining_content, {'items': items})
    assert data == {'LANGUAGE': 'English', 'RUNTIME': '144 minutes'}


def test_generated_key_none_should_be_excluded(shining_content):
    items = [
        {
            'foreach': '//div[@class="info"]',
            'key': {'path': './foo/text()'},
            'value': {'path': './p/text()'}
        }
    ]
    data = scrape(shining_content, {'items': items})
    assert data == {}


def test_tree_should_be_preprocessable(shining_content):
    pre = [{'op': 'set_text', 'path': '//ul[@class="genres"]/li', "text": 'Foo'}]
    items = [{'key': 'genres',
              'value': {
                  'foreach': '//ul[@class="genres"]/li',
                  'path': './text()'
              }}]
    data = scrape(shining_content, {'items': items, 'pre': pre})
    assert data == {'genres': ['Foo', 'Foo']}


def test_section_should_set_root_for_queries(shining_content):
    items = [{'key': 'director',
              'value': {
                  'section': '//div[@class="director"]//a',
                  'items': [{'key': 'name',
                             'value': {'path': './text()'}},
                            {'key': 'link',
                             'value': {'path': './@href'}}]
              }}]
    data = scrape(shining_content, {'items': items})
    assert data == {'director': {'link': '/people/1', 'name': 'Stanley Kubrick'}}


def test_section_no_roots_should_return_empty_result(shining_content):
    items = [{'key': 'director',
              'value': {
                  'section': '//foo',
                  'items': [{'key': 'name',
                             'value': {'path': './text()'}}]
              }}]
    data = scrape(shining_content, {'items': items})
    assert data == {}


def test_section_multiple_roots_should_raise_error(shining_content):
    with raises(ValueError):
        items = [{'key': 'director',
                  'value': {
                      'section': '//div',
                      'items': [{'key': 'name',
                                 'value': {'path': './text()'}}]
                  }}]
        scrape(shining_content, {'items': items})



from __future__ import absolute_import, division, print_function, unicode_literals

from pytest import raises

from piculet import Path, Rule, Rules, build_tree, reducers, transformers


def test_no_rules_should_return_empty_result(shining):
    data = Rules([]).extract(shining)
    assert data == {}


def test_extracted_value_should_be_reduced(shining):
    rules = [Rule(key='title',
                  extractor=Path('//title/text()', reduce=reducers.first))]
    data = Rules(rules).extract(shining)
    assert data == {'title': 'The Shining'}


def test_default_reducer_should_be_concat(shining):
    rules = [Rule(key='full_title',
                  extractor=Path('//h1//text()'))]
    data = Rules(rules).extract(shining)
    assert data == {'full_title': 'The Shining (1980)'}


def test_added_reducer_should_be_usable(shining):
    reducers.register('second', lambda x: x[1])
    rules = [Rule(key='year',
                  extractor=Path('//h1//text()', reduce=reducers.second))]
    data = Rules(rules).extract(shining)
    assert data == {'year': '1980'}


def test_reduce_by_lambda_should_be_ok(shining):
    rules = [Rule(key='title',
                  extractor=Path('//title/text()', reduce=lambda xs: xs[0]))]
    data = Rules(rules).extract(shining)
    assert data == {'title': 'The Shining'}


def test_reduced_value_should_be_transformable(shining):
    rules = [Rule(key='year',
                  extractor=Path('//span[@class="year"]/text()', transform=int))]
    data = Rules(rules).extract(shining)
    assert data == {'year': 1980}


def test_added_transformer_should_be_usable(shining):
    transformers.register('year25', lambda x: int(x) + 25)
    rules = [Rule(key='year',
                  extractor=Path('//span[@class="year"]/text()', transform=transformers.year25))]
    data = Rules(rules).extract(shining)
    assert data == {'year': 2005}


def test_multiple_rules_should_generate_multiple_items(shining):
    rules = [Rule(key='title',
                  extractor=Path('//title/text()')),
             Rule('year',
                  extractor=Path('//span[@class="year"]/text()', transform=int))]
    data = Rules(rules).extract(shining)
    assert data == {'title': 'The Shining', 'year': 1980}


def test_item_with_no_data_should_be_excluded(shining):
    rules = [Rule(key='title',
                  extractor=Path('//title/text()')),
             Rule(key='foo',
                  extractor=Path('//foo/text()'))]
    data = Rules(rules).extract(shining)
    assert data == {'title': 'The Shining'}


def test_item_with_empty_str_value_should_be_included():
    content = '<root><foo val=""/></root>'
    rules = [Rule(key='foo',
                  extractor=Path('//foo/@val'))]
    data = Rules(rules).extract(build_tree(content))
    assert data == {'foo': ''}


def test_item_with_zero_value_should_be_included():
    content = '<root><foo val="0"/></root>'
    rules = [Rule(key='foo',
                  extractor=Path('//foo/@val', transform=int))]
    data = Rules(rules).extract(build_tree(content))
    assert data == {'foo': 0}


def test_item_with_false_value_should_be_included():
    content = '<root><foo val=""/></root>'
    rules = [Rule(key='foo',
                  extractor=Path('//foo/@val', transform=bool))]
    data = Rules(rules).extract(build_tree(content))
    assert data == {'foo': False}


def test_multivalued_item_should_be_list(shining):
    rules = [Rule(key='genres',
                  extractor=Path(foreach='//ul[@class="genres"]/li',
                                 path='./text()'))]
    data = Rules(rules).extract(shining)
    assert data == {'genres': ['Horror', 'Drama']}


def test_multivalued_items_should_be_transformable(shining):
    rules = [Rule(key='genres',
                  extractor=Path(foreach='//ul[@class="genres"]/li',
                                 path='./text()', transform=transformers.lower))]
    data = Rules(rules).extract(shining)
    assert data == {'genres': ['horror', 'drama']}


def test_empty_values_should_be_excluded_from_multivalued_item_list(shining):
    rules = [Rule(key='foos',
                  extractor=Path(foreach='//ul[@class="foos"]/li',
                                 path='./text()'))]
    data = Rules(rules).extract(shining)
    assert data == {}


def test_subrules_should_generate_subitems(shining):
    rules = [
        Rule(key='director',
             extractor=Rules(
                 rules=[
                     Rule(key='name',
                          extractor=Path('//div[@class="director"]//a/text()')),
                     Rule(key='link',
                          extractor=Path('//div[@class="director"]//a/@href'))
                 ]))
    ]
    data = Rules(rules).extract(shining)
    assert data == {'director': {'link': '/people/1', 'name': 'Stanley Kubrick'}}


def test_multivalued_subrules_should_generate_list_of_subitems(shining):
    rules = [
        Rule(key='cast',
             extractor=Rules(
                 foreach='//table[@class="cast"]/tr',
                 rules=[
                     Rule(key='name',
                          extractor=Path('./td[1]/a/text()')),
                     Rule(key='character',
                          extractor=Path('./td[2]/text()'))
                 ]))
    ]
    data = Rules(rules).extract(shining)
    assert data == {'cast': [
        {'character': 'Jack Torrance', 'name': 'Jack Nicholson'},
        {'character': 'Wendy Torrance', 'name': 'Shelley Duvall'}
    ]}


def test_subitems_should_be_transformable(shining):
    rules = [
        Rule(key='cast',
             extractor=Rules(
                 foreach='//table[@class="cast"]/tr',
                 rules=[
                     Rule(key='name',
                          extractor=Path('./td[1]/a/text()')),
                     Rule(key='character',
                          extractor=Path('./td[2]/text()'))
                 ],
                 transform=lambda x: '%(name)s as %(character)s' % x
             ))
    ]
    data = Rules(rules).extract(shining)
    assert data == {'cast': ['Jack Nicholson as Jack Torrance',
                             'Shelley Duvall as Wendy Torrance']}


def test_key_should_be_generatable_using_path(shining):
    rules = [Rule(foreach='//div[@class="info"]',
                  key=Path('./h3/text()'),
                  extractor=Path('./p/text()'))]
    data = Rules(rules).extract(shining)
    assert data == {'Language:': 'English', 'Runtime:': '144 minutes'}


def test_generated_key_should_be_normalizable(shining):
    rules = [Rule(foreach='//div[@class="info"]',
                  key=Path('./h3/text()', reduce=reducers.normalize),
                  extractor=Path('./p/text()'))]
    data = Rules(rules).extract(shining)
    assert data == {'language': 'English', 'runtime': '144 minutes'}


def test_generated_key_should_be_transformable(shining):
    rules = [Rule(foreach='//div[@class="info"]',
                  key=Path('./h3/text()', reduce=reducers.normalize,
                           transform=lambda x: x.upper()),
                  extractor=Path('./p/text()'))]
    data = Rules(rules).extract(shining)
    assert data == {'LANGUAGE': 'English', 'RUNTIME': '144 minutes'}


def test_generated_key_none_should_be_excluded(shining):
    rules = [Rule(foreach='//div[@class="info"]',
                  key=Path('./foo/text()'),
                  extractor=Path('./p/text()'))]
    data = Rules(rules).extract(shining)
    assert data == {}


def test_section_should_set_root_for_queries(shining):
    rules = [
        Rule(key='director',
             extractor=Rules(
                 section='//div[@class="director"]//a',
                 rules=[
                     Rule(key='name',
                          extractor=Path('./text()')),
                     Rule(key='link',
                          extractor=Path('./@href'))
                 ]))
    ]
    data = Rules(rules).extract(shining)
    assert data == {'director': {'link': '/people/1', 'name': 'Stanley Kubrick'}}


def test_section_no_roots_should_return_empty_result(shining):
    rules = [
        Rule(key='director',
             extractor=Rules(
                 section='//foo',
                 rules=[
                     Rule(key='name',
                          extractor=Path('./text()'))
                 ]))
    ]
    data = Rules(rules).extract(shining)
    assert data == {}


def test_section_multiple_roots_should_raise_error(shining):
    with raises(ValueError):
        rules = [
            Rule(key='director',
                 extractor=Rules(
                     section='//div',
                     rules=[
                         Rule(key='name',
                              extractor=Path('./text()'))
                     ]))
        ]
        Rules(rules).extract(shining)



from __future__ import absolute_import, division, print_function, unicode_literals

from pytest import config, mark, raises

import json
import logging
import os
import sys
from io import StringIO

from pkg_resources import get_distribution

import piculet


if sys.version_info.major < 3:
    import mock
else:
    from unittest import mock


base_dir = os.path.dirname(__file__)
wikipedia_spec = os.path.join(base_dir, '..', 'examples', 'wikipedia.json')


def test_help_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        piculet.main(argv=['piculet', '--help'])
    out, err = capsys.readouterr()
    assert out.startswith('usage: ')


def test_version_should_print_version_number_and_exit(capsys):
    with raises(SystemExit):
        piculet.main(argv=['piculet', '--version'])
    out, err = capsys.readouterr()
    assert 'piculet ' + get_distribution('piculet').version + '\n' in {out, err}


def test_no_command_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        piculet.main(argv=['piculet'])
    out, err = capsys.readouterr()
    assert err.startswith('usage: ')
    assert ('required: command' in err) or ('too few arguments' in err)


def test_invalid_command_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        piculet.main(argv=['piculet', 'foo'])
    out, err = capsys.readouterr()
    assert err.startswith('usage: ')
    assert ('invalid choice: \'foo\'' in err) or ('invalid choice: u\'foo\'' in err)


def test_unrecognized_arguments_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        piculet.main(argv=['piculet', '--foo', 'h2x', ''])
    out, err = capsys.readouterr()
    assert err.startswith('usage: ')
    assert 'unrecognized arguments: --foo' in err


def test_debug_mode_should_print_debug_messages(caplog):
    caplog.set_level(logging.DEBUG)
    with mock.patch('sys.stdin', StringIO('<html></html>')):
        piculet.main(argv=['piculet', '--debug', 'h2x', '-'])
    assert caplog.record_tuples[0][-1] == 'running in debug mode'


def test_h2x_no_input_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        piculet.main(argv=['piculet', 'h2x'])
    out, err = capsys.readouterr()
    assert err.startswith('usage: ')
    assert ('required: file' in err) or ('too few arguments' in err)


@mark.skipif(sys.platform not in {'linux', 'linux2'},
             reason='/dev/shm only available on linux')
def test_h2x_should_read_given_file(capsys):
    content = '<html></html>'
    with open('/dev/shm/test.html', 'w') as f:
        f.write(content)
    piculet.main(argv=['piculet', 'h2x', '/dev/shm/test.html'])
    out, err = capsys.readouterr()
    os.unlink('/dev/shm/test.html')
    assert out == content


def test_h2x_should_read_stdin_when_input_is_dash(capsys):
    content = '<html></html>'
    with mock.patch('sys.stdin', StringIO(content)):
        piculet.main(argv=['piculet', 'h2x', '-'])
    out, err = capsys.readouterr()
    assert out == content


def test_scrape_no_url_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        piculet.main(argv=['piculet', 'scrape', '-s', wikipedia_spec])
    out, err = capsys.readouterr()
    assert err.startswith('usage: ')
    assert ('required: document' in err) or ('too few arguments' in err)


def test_scrape_no_spec_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        piculet.main(argv=['piculet', 'scrape', 'http://www.foo.com/'])
    out, err = capsys.readouterr()
    assert err.startswith('usage: ')
    assert ('required: -s' in err) or ('--spec is required' in err)


def test_scrape_missing_spec_file_should_fail_and_exit(capsys):
    with raises(SystemExit):
        piculet.main(argv=['piculet', 'scrape', 'http://www.foo.com/', '-s', 'foo.json'])
    out, err = capsys.readouterr()
    assert 'No such file or directory: ' in err


def test_scrape_local_should_scrape_given_file(capsys):
    dirname = os.path.join(os.path.dirname(__file__), '..', 'examples')
    shining = os.path.join(dirname, 'shining.html')
    spec = os.path.join(dirname, 'movie.json')
    piculet.main(argv=['piculet', 'scrape', shining, '-s', spec])
    out, err = capsys.readouterr()
    data = json.loads(out)
    assert data['title'] == 'The Shining'


@mark.skipif(not config.getvalue('--cov'),
             reason='takes unforeseen amount of time')
def test_scrape_should_scrape_given_url(capsys):
    piculet.main(argv=['piculet', 'scrape', 'https://en.wikipedia.org/wiki/David_Bowie',
                       '-s', wikipedia_spec, '--html'])
    out, err = capsys.readouterr()
    data = json.loads(out)
    assert data['name'] == 'David Bowie'



from __future__ import absolute_import, division, print_function, unicode_literals

from piculet import reducers


def test_reducer_first_should_return_first_item():
    assert reducers.first(['a', 'b', 'c']) == 'a'


def test_reducer_concat_should_return_concatenated_items():
    assert reducers.concat(['a', 'b', 'c']) == 'abc'


def test_reducer_clean_should_remove_extra_space():
    assert reducers.clean(['  a ', '   b', ' c ']) == 'a b c'


def test_reducer_clean_should_treat_nbsp_as_space():
    assert reducers.clean(['  a ', ' \xa0  b', ' c ']) == 'a b c'


def test_reducer_normalize_should_convert_to_lowercase():
    assert reducers.normalize(['A', 'B', 'C']) == 'abc'


def test_reducer_normalize_should_remove_nonalphanumeric_characters():
    assert reducers.normalize(['a+', '?b7', '{c}']) == 'ab7c'


def test_reducer_normalize_should_keep_underscores():
    assert reducers.normalize(['a_', 'b', 'c']) == 'a_bc'


def test_reducer_normalize_should_replace_spaces_with_underscores():
    assert reducers.normalize(['a', ' b', 'c']) == 'a_bc'
